#region Copyright Syncfusion Inc. 2001 - 2014
// Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Syncfusion.Windows.Controls.Grid;
using Syncfusion.Windows.Shared;
using System.Collections.ObjectModel;
using System.Windows.Interactivity;

namespace TileViewItemSample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class DataGridSample2 : UserControl
    {
        public DataGridSample2()
        {
            InitializeComponent();            
        }
    }


    class ViewModel
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="ViewModel"/> class.
        /// </summary>
        public ViewModel()
        {
            CountryInfo = GetCountryInfo();
        }

        /// <summary>
        /// Gets the country details.
        /// </summary>
        /// <returns></returns>
        private ObservableCollection<CountryDetails> GetCountryInfo()
        {
            ObservableCollection<CountryDetails> ConDetails = new ObservableCollection<CountryDetails>();
           
            
            ConDetails.Add(new CountryDetails()
            {
                CountryName = "India",
           
                Image = @"..\..\Images\India.jpg",
                Population = 1184639000,
              

            });
            ConDetails.Add(new CountryDetails()
            {
                CountryName = "USA",
              
                Image = @"..\..\Images\USA.jpg",
                Population = 309975000,
              

            });
            ConDetails.Add(new CountryDetails()
            {
                CountryName = "Indonesia",
              
                Image = @"..\..\Images\Indonesia.jpg",
                Population = 234181400,
                

            });
            ConDetails.Add(new CountryDetails()
            {
                CountryName = "Brazil",
              
                Image = @"..\..\Images\Brazil.jpg",
                Population = 193364000,
                

            });

            return ConDetails;
        }

        private ObservableCollection<CountryDetails> _countryInfo;

        /// <summary>
        /// Gets or sets the country info.
        /// </summary>
        /// <value>The country info.</value>
        public ObservableCollection<CountryDetails> CountryInfo
        {
            get
            {
                return _countryInfo;
            }
            set
            {
                _countryInfo = value;
            }
        }
    }

    public class CountryDetails : NotificationObject
    {
        private string _countryName;
        /// <summary>
        /// Gets or sets the name of the country.
        /// </summary>
        /// <value>The name of the country.</value>
        public string CountryName
        {
            get
            {
                return _countryName;
            }
            set
            {
                _countryName = value;
                RaisePropertyChanged("CountryName");
            }
        }

        private double _population;
        /// <summary>
        /// Gets or sets the population.
        /// </summary>
        /// <value>The population.</value>
        public double Population
        {
            get
            {
                return _population;
            }
            set
            {
                _population = value;
                RaisePropertyChanged("Population");
            }
        }

        private double _area;
        /// <summary>
        /// Gets or sets the area.
        /// </summary>
        /// <value>The area.</value>
        public double Area
        {
            get
            {
                return _area;
            }
            set
            {
                _area = value;
                RaisePropertyChanged("Area");
            }
        }

        private string _image;
        /// <summary>
        /// Gets or sets the image.
        /// </summary>
        /// <value>The image.</value>
        public string Image
        {
            get
            {
                return _image;
            }
            set
            {
                _image = value;
                RaisePropertyChanged("Image");
            }
        }

        private int _rankInPopulation;
        /// <summary>
        /// Gets or sets the rank in population.
        /// </summary>
        /// <value>The rank in population.</value>
        public int RankInPopulation
        {
            get
            {
                return _rankInPopulation;
            }
            set
            {
                _rankInPopulation = value;
                RaisePropertyChanged("RankInPopulation");
            }
        }

        private int _rankinArea;
        /// <summary>
        /// Gets or sets the rankin area.
        /// </summary>
        /// <value>The rankin area.</value>
        public int RankinArea
        {
            get
            {
                return _rankinArea;
            }
            set
            {
                _rankinArea = value;
                RaisePropertyChanged("RankinArea");
            }
        }

        private double _populationPercentage;
        /// <summary>
        /// Gets or sets the population percentage.
        /// </summary>
        /// <value>The population percentage.</value>
        public double PopulationPercentage
        {
            get
            {
                return _populationPercentage;
            }
            set
            {
                _populationPercentage = value;
                RaisePropertyChanged("PopulationPercentage");
            }
        }

        private double _populationDensity;
        /// <summary>
        /// Gets or sets the population density.
        /// </summary>
        /// <value>The population density.</value>
        public double PopulationDensity
        {
            get
            {
                return _populationDensity;
            }
            set
            {
                _populationDensity = value;
                RaisePropertyChanged("PopulationDensity");
            }
        }
    }

    public class StringToImageConverter : IValueConverter
    {
        /// <summary>
        /// Converts a value.
        /// </summary>
        /// <param name="value">The value produced by the binding source.</param>
        /// <param name="targetType">The type of the binding target property.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value is CountryDetails)
            {
                var record = (CountryDetails)value;
                if (string.IsNullOrEmpty(record.Image))
                    return new BitmapImage(new Uri(@"..\..\Images\NotAvailable.jpg", UriKind.Relative));
                return new BitmapImage(new Uri(record.Image, UriKind.Relative));
            }
            else if (value is string && value.ToString().Contains(@"..\..\Images"))
            {
                return value;
            }
            else
            {
                string imagename = value as string;
                if (string.IsNullOrEmpty(imagename))
                    imagename = @"..\..\Images\NotAvailable.jpg";

                return @"..\..\Images\" + imagename + @".jpg";
            }
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    class SummaryCalculationBehavior : Behavior<GridDataControl>
    {
        #region InternalVariables
        /// <summary>
        /// Dictionary to maintain the UnBoundColumnValues based on the Product name for dataGrid1
        /// </summary>
        Dictionary<string, bool> UnboundColumnDictionary_dataGrid1 = new Dictionary<string, bool>();


        /// <summary>
        /// Gets or sets the price1 summary for dataGrid1
        /// </summary>
        /// <value>The price1 summary.</value>
        public double PopulationSummary { get; set; }

        /// <summary>
        /// Gets or sets the price2 summary for dataGrid1
        /// </summary>
        /// <value>The price2 summary.</value>
        public double AreaSummary { get; set; }

        /// <summary>
        /// Gets or sets the price3 summary for dataGrid1
        /// </summary>
        /// <value>The price3 summary.</value>
        public double PopulationDensitySummary { get; set; }

        /// <summary>
        /// Gets or sets the totoal count  for dataGrid1
        /// </summary>
        /// <value>The totoal count.</value>
        public double TotoalCount_dataGrid1 { get; set; }

        #endregion

        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            ViewModel Model = this.AssociatedObject.DataContext as ViewModel;
            this.AssociatedObject.Model.UnboundRowsCount = 1;
            this.AssociatedObject.ModelLoaded += new EventHandler(AssociatedObject_ModelLoaded);
            for (int i = 0; i < Model.CountryInfo.Count; i++)
            {
                UnboundColumnDictionary_dataGrid1.Add(Model.CountryInfo[i].CountryName, false);
            }
            this.AssociatedObject.Model.QueryCellInfo += Model_QueryCellInfo;
            this.AssociatedObject.CurrentCellChanged += AssociatedObject_CurrentCellChanged;
            this.AssociatedObject.ModelLoaded += new EventHandler(AssociatedObject_ModelLoaded);
        }

        void AssociatedObject_ModelLoaded(object sender, EventArgs e)
        {
            this.AssociatedObject.Model.RowHeights.DefaultLineSize = 40;
        }

        /// <summary>
        /// Handles the CurrentCellChanged event of the AssociatedObject control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="args">The <see cref="Syncfusion.Windows.ComponentModel.SyncfusionRoutedEventArgs"/> instance containing the event data.</param>
        void AssociatedObject_CurrentCellChanged(object sender, Syncfusion.Windows.ComponentModel.SyncfusionRoutedEventArgs args)
        {
            GridDataControl _dataGrid = this.AssociatedObject;
            //Getting the index of VisibleColumn from ColumnIndex of Grid
            var columnIndex = _dataGrid.Model.ResolvePositionToVisibleColumnIndex(_dataGrid.Model.CurrencyManager.CurrentCell.ColumnIndex);
            if (columnIndex >= 0)
            {
                var column = _dataGrid.Model.TableProperties.VisibleColumns[columnIndex];
                var recordIndex = _dataGrid.Model.CurrencyManager.CurrentRecordIndex;
                //Adding condition check to handle Unbound column values alone
                if (column.MappingName == "Status")
                {
                    var checkbx = _dataGrid.Model.CurrencyManager.CurrentCell.Renderer.CurrentCellUIElement;
                    var key = (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).CountryName;

                    bool _containsValue = false;
                    _containsValue = UnboundColumnDictionary_dataGrid1.ContainsKey(key);
                    //Updation the checkbox value in dictionary and calculation the summary value based on the check box state
                    if (_containsValue)
                    {
                        UnboundColumnDictionary_dataGrid1[key] = (bool)(checkbx as CheckBox).IsChecked;
                        if ((bool)(checkbx as CheckBox).IsChecked)
                        {
                            PopulationSummary += (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).Population;
                            AreaSummary += (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).Area;
                            PopulationDensitySummary += (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).PopulationDensity;
                            if (PopulationDensitySummary < 0)
                                PopulationDensitySummary = 0;
                            TotoalCount_dataGrid1 = TotoalCount_dataGrid1 + 1;
                        }
                        else
                        {
                            PopulationSummary -= (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).Population;
                            AreaSummary -= (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).Area;
                            PopulationDensitySummary -= (_dataGrid.Model.View.Records[recordIndex].Data as CountryDetails).PopulationDensity;
                            if (PopulationDensitySummary < 0)
                                PopulationDensitySummary = 0;
                            TotoalCount_dataGrid1 = TotoalCount_dataGrid1 - 1;
                        }

                        //Invalidation the last Unbound row to refresh with  new summary values
                        _dataGrid.Model.InvalidateCell(GridRangeInfo.Row(_dataGrid.Model.RowCount - 1));
                        _dataGrid.Model.InvalidateCell(GridRangeInfo.Col(0));
                    }
                }
            }
        }

        /// <summary>
        /// Handles the QueryCellInfo event of the Model control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Syncfusion.Windows.Controls.Grid.GridQueryCellInfoEventArgs"/> instance containing the event data.</param>
        void Model_QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
        {
            GridDataTableModel _gridDataTableModel = sender as GridDataTableModel;
            GridDataStyleInfo styleinfo = e.Style as GridDataStyleInfo;

            //Updating the Unbound row values based on calculated summary values
            if (styleinfo.CellIdentity.TableCellType == GridDataTableCellType.UnboundRecordCell)
            {
                e.Style.Background = Brushes.LightGray;
                e.Style.Foreground = Brushes.Black;
                e.Style.ReadOnly = true;

                var columnIndex = _gridDataTableModel.ResolvePositionToVisibleColumnIndex(e.Style.ColumnIndex);
                if (columnIndex >= 0)
                {
                    var column = _gridDataTableModel.TableProperties.VisibleColumns[columnIndex];

                    if (column.MappingName == "Population")
                    {
                        e.Style.CellValue = PopulationSummary + ".00";
                    }
                    else if (column.MappingName == "Area")
                    {
                        e.Style.CellValue = AreaSummary + ".00";
                    }
                    else if (column.MappingName == "PopulationDensity")
                    {
                        e.Style.CellValue = PopulationDensitySummary + "";
                    }
                    else if (column.MappingName == "Status")
                        e.Style.CellValue = "Total count = " + TotoalCount_dataGrid1;
                    e.Style.HorizontalAlignment = System.Windows.HorizontalAlignment.Right;
                }
            }
            //Updating the Unbound column value based on dictionary values
            if (styleinfo.CellIdentity.TableCellType == GridDataTableCellType.UnboundColumnCell)
            {
                var dictkey = (_gridDataTableModel.View.Records[styleinfo.CellIdentity.RecordIndex].Data as CountryDetails).CountryName;
                e.Style.CellValue = UnboundColumnDictionary_dataGrid1[dictkey];
            }
        }


        /// <summary>
        /// Called when [detaching].
        /// </summary>
        protected override void OnDetaching()
        {
            this.AssociatedObject.Model.QueryCellInfo -= Model_QueryCellInfo;
            this.AssociatedObject.CurrentCellChanged -= AssociatedObject_CurrentCellChanged;
            this.AssociatedObject.ModelLoaded -= new EventHandler(AssociatedObject_ModelLoaded);
        }
    }
}
